package app;

public class MessageError {
    public enum Code {
        SUCCESS(0, "Success"),
        UNKNOWN_ERROR(1, "Unknown error"),
        INVALID_PACKET_FORMAT(2, "Invalid packet format"),
		INVALID_PACKET_ID(3, "Invalid packet number"),
		INVALID_SEQUENCE_ID(4, "Invalid sequence number"),
		INVALID_MESSAGE_SIZE(5, "Invalid message size"),
		INVALID_MESSAGE_TYPE(6, "Unsupported message ID"),
        CRC_ERROR(7, "CRC error"),
        DATA_OVERFLOW(8, "Data overflow"),
        PACKET_LOST(9, "Packet lost"),
        CONNECTION_FAILED(10, "Connection failed"),
        DISCONNECTION_FAILED(11, "Disconnection failed"),
        CONNECTION_LOST(12, "Connection lost"),
        TIMEOUT(13, "Operation timed out"),
        AUTHENTICATION_FAILED(14, "Authentication failed"),
        PERMISSION_DENIED(15, "Permission denied"),
        OPERATION_CANCELLED(16, "Operation cancelled"),
        OPERATION_FAILURE(17, "Operation failed"),
        INSUFFICIENT_RESOURCES(18, "Insufficient resources"),
        MEMORY_ALLOCATION_ERROR(19, "Memory allocation error"),
        INVALID_NOTIFY(20, "Invalid notify"),
        INVALID_PASSWORD_FORMAT(21, "Invalid password format"),
        INVALID_TIMESTAMP(22, "Invalid timestamp"),
        PAIRING_ERROR(23, "Pairing error"),
		INVALID_PARAMETERS(24, "Invalid parameters"),
		INVALID_PACKET_SIZE(25, "Invalid packet size"),
		INVALID_SECRET(26, "Invalid secret string"),
		DEVICE_NOT_FOUND(27, "Device not found"),
		DEVICE_BUSY(28, "Device busy"),
		UNSUPPORTED_FEATURE(29, "Unsupported feature"),
		STATE_ERROR(30, "State error"),
		HARDWARE_FAILURE(31, "Hardware failure"),
        SOFTWARE_ERROR(32, "Software error"),
        UNSUPPORTED_VERSION(33, "Unsupported version"),
        INCOMPATIBLE_DEVICE(34, "Incompatible device"),
        INIT_ERROR(35, "Device init error");

        private final int code;
        private final String description;

        Code(int code, String description) {
            this.code = code;
            this.description = description;
        }

        public int getCode() {
            return code;
        }

        public String getDescription() {
            return description;
        }

        @Override
        public String toString() {
            return code + ": " + description;
        }
    }

    public static Code fromCode(int code) {
        for (Code errorCode : Code.values()) {
            if (errorCode.getCode() == code) {
                return errorCode;
            }
        }
        return Code.UNKNOWN_ERROR;
    }
}

