﻿using System;
using System.Threading;
using System.IO.Ports;
using WatcherCommons.Interfaces;
using ArduinoSimpleTriggerProtocol.Properties;

namespace ArduinoSimpleTriggerProtocol
{
    public class ArduinoTrigger : ITrigger
    {
        private string _port = "";
        private int _baudrate = 9600;
        private Thread _thTrigger = null;
        private bool _bRunning = false;
        public ArduinoTrigger()
        {
        }
        public event EventHandler OnTriggerFired = null;
        public string ConnectionString
        {
            get
            {
                return string.Join(";", new string[] { "port=" + _port, "baudrate=" + _baudrate.ToString() });
            }
            set
            {
                foreach (string s in value.Split(';'))
                {
                    string[] p = s.Split('=');
                    if (string.Compare(p[0], "port", true) == 0)
                    {
                        _port = p[1];
                    }
                    else if (string.Compare(p[0], "baudrate", true) == 0)
                    {
                        _baudrate = int.Parse(p[1]);
                    }
                }
            }
        }
        public void Initialize()
        {
            OnTriggerFired = null;
            Stop();
        }
        public void Start()
        {
            if (_thTrigger == null)
            {
                try
                {
                    using (SerialPort p = new SerialPort(_port))
                    {
                        p.BaudRate = _baudrate;
                        p.ReadTimeout = 1000;
                        p.Open();
                        p.ReadExisting();
                        p.Close();
                    }
                }
                catch
                {
                }
                _bRunning = true;
                _thTrigger = new Thread(TriggerTest);
                _thTrigger.Start();
            }
        }
        public void Stop()
        {
            try
            {
                if (_thTrigger != null)
                {
                    _bRunning = false;
                    _thTrigger.Abort();
                    while (_thTrigger.IsAlive) ;
                    _thTrigger = null;
                }
            }
            catch
            {
            }
        }
        private void TriggerTest()
        {
            while (_bRunning && (OnTriggerFired != null))
            {
                try
                {
                    using (SerialPort p = new SerialPort(_port))
                    {
                        p.BaudRate = _baudrate;
                        p.ReadTimeout = 1000;
                        p.Open();
                        int val = p.ReadByte();
                        p.Close();
                        if (val == 1)
                        {
                            OnTriggerFired?.Invoke(this, EventArgs.Empty);
                            Thread.Sleep(2000);
                        }
                    }
                }
                catch (Exception ex)
                {
                    string msg = ex.Message;
                }
            }
            _thTrigger = null;
        }
        public override string ToString()
        {
            return Resources.ID_PROTOCOL;
        }
    }
}
