﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using WatcherCommons.Interfaces;
using WatcherCommons.Data;
using AzureBlobProtocol.Properties;

namespace AzureBlobProtocol
{
    public class AzureBlobManager : IStorageManager
    {
        public AzureBlobManager()
        {
        }
        public string ConnsecionString { get; set; }
        public string ContainerPath { get; set; }
        public void UploadFile(string filename, Stream s)
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            ICloudBlob blob = container.GetBlockBlobReference(filename);
            blob.UploadFromStream(s);
        }
        public void DownloadFile(string filename, Stream s)
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            ICloudBlob blob = container.GetBlockBlobReference(filename);
            blob.DownloadToStream(s);
        }
        public void DeleteFile(string filename)
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            ICloudBlob blob = container.GetBlockBlobReference(filename);
            blob.Delete();
        }
        public bool ExistsFile(string filename)
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            foreach (IListBlobItem ib in container.ListBlobs(filename, true))
            {
                return true;
            }
            return false;
        }
        public IEnumerable<string> ListFiles(string model)
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            foreach (IListBlobItem ib in container.ListBlobs(model, true))
            {
                yield return ib.Uri.Segments[ib.Uri.Segments.Length - 1];
            }
        }
        public IEnumerable<ControlCommand> GetCommands()
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            foreach (IListBlobItem ib in container.ListBlobs("cmd.", true))
            {
                MemoryStream ms = new MemoryStream();
                DownloadFile(ib.Uri.Segments[ib.Uri.Segments.Length - 1], ms);
                DeleteFile(ib.Uri.Segments[ib.Uri.Segments.Length - 1]);
                ControlCommand cc = ControlCommand.FromJSON(ms);
                yield return cc;
            }
        }
        public IEnumerable<List<CameraInfo>> GetRequests()
        {
            CloudStorageAccount acct = CloudStorageAccount.Parse(ConnsecionString);
            CloudBlobClient client = acct.CreateCloudBlobClient();
            CloudBlobContainer container = client.GetContainerReference(ContainerPath);
            container.CreateIfNotExists();
            foreach (IListBlobItem ib in container.ListBlobs("req.", true))
            {
                MemoryStream ms = new MemoryStream();
                DownloadFile(ib.Uri.Segments[ib.Uri.Segments.Length - 1], ms);
                DeleteFile(ib.Uri.Segments[ib.Uri.Segments.Length - 1]);
                List<CameraInfo> lci = CameraInfo.FromJSON(ms);
                yield return lci;
            }
        }
        public void SendResponse(List<CameraInfo> resp)
        {
            MemoryStream ms = new MemoryStream();
            CameraInfo.ToJSON(ms, resp);
            ms.Position = 0;
            UploadFile("resp." + resp[0].ClientID + ".json", ms);
        }
        public override string ToString()
        {
            return Resources.ID_PROTOCOL;
        }
    }
}
