﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using WatcherCommons.Interfaces;
using WatcherCommons.Data;
using DropBoxProtocol.Properties;

namespace DropBoxProtocol
{
    public class DropBoxStorage : IStorageManager
    {
        public DropBoxStorage()
        {
        }
        public string ConnsecionString { get; set; }
        public string ContainerPath { get; set; }
        public void UploadFile(string filename, Stream s)
        {
            FileStream fs = null;
            try
            {
                fs = new FileStream(Path.Combine(ContainerPath, filename), FileMode.Create, FileAccess.Write);
                s.CopyTo(fs);
                fs.Flush();
            }
            finally
            {
                if (fs != null)
                {
                    fs.Close();
                }
            }
        }
        public void DownloadFile(string filename, Stream s)
        {
            FileStream fs = null;
            try
            {
                fs = new FileStream(Path.Combine(ContainerPath, filename), FileMode.Open, FileAccess.Read, FileShare.Read);
                fs.CopyTo(s);
                fs.Flush();
            }
            finally
            {
                if (fs != null)
                {
                    fs.Close();
                }
            }
        }
        public void DeleteFile(string filename)
        {
            File.Delete(Path.Combine(ContainerPath, filename));
        }
        public bool ExistsFile(string filename)
        {
            return File.Exists(Path.Combine(ContainerPath, filename));
        }
        public IEnumerable<string> ListFiles(string model)
        {
            foreach (string f in Directory.EnumerateFiles(ContainerPath))
            {
                if (string.IsNullOrEmpty(model) || Path.GetFileName(f).StartsWith(model))
                {
                    yield return Path.GetFileName(f);
                }
            }
        }
        public IEnumerable<ControlCommand> GetCommands()
        {
            foreach (string f in ListFiles("cmd_"))
            {
                MemoryStream ms = new MemoryStream();
                DownloadFile(f, ms);
                ControlCommand cc = ControlCommand.FromJSON(ms);
                if (cc != null)
                {
                    DeleteFile(f);
                    yield return cc;
                }
                else
                {
                    yield break;
                }
            }
        }
        public IEnumerable<List<CameraInfo>> GetRequests()
        {
            foreach (string f in ListFiles("req_"))
            {
                MemoryStream ms = new MemoryStream();
                DownloadFile(f, ms);
                List<CameraInfo> lci = CameraInfo.FromJSON(ms);
                if ((lci != null) && (lci.Count > 0))
                {
                    DeleteFile(f);
                    yield return lci;
                }
                else
                {
                    yield break;
                }
            }
        }
        public void SendResponse(List<CameraInfo> resp)
        {
            MemoryStream ms = new MemoryStream();
            CameraInfo.ToJSON(ms, resp);
            ms.Position = 0;
            UploadFile("resp_" + resp[0].ClientID + ".json", ms);
        }
        public override string ToString()
        {
            return Resources.ID_PROTOCOL;
        }
    }
}
