﻿using System;
using System.Collections.Specialized;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using NetWaveProtocol.Properties;
using WatcherCommons.Interfaces;

namespace NetWaveProtocol
{
    // Proporciona soporte para configurar los parámetros de la cámara, como brillo, contraste y movimiento
    // Provides support to configure camera settings, such as brightness, contrast and movement
    public partial class CameraConfiguration : Form, ICameraSetupManager
    {
        private bool _bMoveUp = false;
        private bool _bMoveDown = false;
        private bool _bMoveLeft = false;
        private bool _bMoveRight = false;

        public CameraConfiguration()
        {
            InitializeComponent();
            Text = UIResources.TTL_CAMERACONTROL;
            LBrightness.Text = UIResources.LAB_BRIGHTNESS;
            LContrast.Text = UIResources.LAB_CONTRAST;
            CBFlip.Text = UIResources.LAB_FLIP;            
        }
        public event EventHandler OnFrameSizeChanged = null;
        public void ReloadSettings()
        {
            try
            {
                Cursor = Cursors.WaitCursor;
                GetCurrentSettings();
                Cursor = Cursors.Default;
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Default;
                MessageBox.Show(ex.Message);
            }
        }
        public string URL { get; set; }
        public string UserName { get; set; }
        public string Password { get; set; }
        public int Resolution { get; set; }
        public NetWaveCamera Camera { get; set; }
        
        // Soporte para comados cgi que devuelven texto como respuesta
        // Support for cgi commands that return text as response
        private string RequestCommand(string url)
        {
            string userpwd = UserName + ":" + Password;
            UTF8Encoding enc = new UTF8Encoding();
            HttpWebRequest wrq = (HttpWebRequest)HttpWebRequest.Create(url);
            wrq.Method = "GET";
            // Proprciona un timeot corto a las operaciones de control para dar prioridad a la captura de imágenes
            // Provide a short timeot to the control operations to priorize the image capture
            wrq.Timeout = 2000;
            // Autenticación básica por encabezados http
            // Http headers basic Authentication
            wrq.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(enc.GetBytes(userpwd)));
            WebResponse wr = wrq.GetResponse();
            Stream receiveStream = wr.GetResponseStream();
            StreamReader reader = new StreamReader(receiveStream, Encoding.UTF8);
            string response = reader.ReadToEnd();
            wr.Close();
            return response;
        }
        // Decodifica una respuesta con varias líneas con estructura var <variable>=<valor>;
        // Decodes a response with multiple lines with structure var <variable>=<value>;
        private StringDictionary DecodeResponse(string response)
        {
            StringDictionary dict = new StringDictionary();
            StringReader rdr = new StringReader(response);
            string line = rdr.ReadLine();
            while (line != null)
            {
                // Elimina las partes sobrantes de la línea
                // Removes unimportant parts of the line 
                line = line.Replace("var ", "");
                line = line.Replace(";", "");
                // Separa la variable de su valor
                // Separates variable name from its value
                string[] parts = line.Split('=');
                // Crea una entrada en el diccionario para la variable
                // Create an entry in the dictionary for the variable
                dict[parts[0]] = parts[1];
                line = rdr.ReadLine();
            }
            return dict;
        }
        // Obtiene los valores actuales de brillo, contraste y posición de pantalla
        // Gets the current settings of brightness, contrast and screen position
        private void GetCurrentSettings()
        {
            try
            {
                // Otra forma de autenticar la petición es mediante parámetros
                // Another way to authenticate the request is by means of parameters
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "get_camera_params.cgi?user=" + UserName + "&pwd=" + Password;
                string response = RequestCommand(url);
                StringDictionary dict = DecodeResponse(response);
                TBBrightness.Value = Convert.ToInt32(dict["brightness"]);
                TBContrast.Value = Convert.ToInt32(dict["contrast"]);
                CBFlip.Checked = dict["flip"] == "2";
                rb320.Checked = dict["resolution"] == "8";
                rb640.Checked = dict["resolution"] == "32";
                Camera.SetResolution(int.Parse(dict["resolution"]));
                OnFrameSizeChanged?.Invoke(Camera, EventArgs.Empty);
            }
            catch
            {                
            }
        }

        private void TBBrightness_Scroll(object sender, EventArgs e)
        {
            try
            {
                // Comando para modificar el brillo, entre 0 y 255
                // Command to change the brightness, from 0 to 255
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "camera_control.cgi?param=1&value=" + TBBrightness.Value.ToString();
                RequestCommand(url);
            }
            catch
            {
            }
        }

        private void TBContrast_Scroll(object sender, EventArgs e)
        {
            try
            {
                // Comando para modificar el contraste, entre 0 y 6
                // Command to change the contrast, between 0 and 6
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "camera_control.cgi?param=2&value=" + TBContrast.Value.ToString();
                RequestCommand(url);
            }
            catch
            {
            }
        }

        private void CBFlip_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                // Comando para invertir horizontalmente la imagen capturada
                // Command to reverse the captured image horizontally
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "camera_control.cgi?param=5&value=" + (CBFlip.Checked ? "2" : "0");
                RequestCommand(url);
            }
            catch
            {
            }
        }
        private void rb640_CheckedChanged(object sender, EventArgs e)
        {
            if (rb640.Checked)
            {
                try
                {
                    // Comando para invertir horizontalmente la imagen capturada
                    // Command to reverse the captured image horizontally
                    string url = URL + (URL.EndsWith("/") ? "" : "/") + "camera_control.cgi?param=0&value=32";
                    RequestCommand(url);
                    rb320.Checked = false;
                    Resolution = 32;
                    Camera.SetResolution(32);
                    OnFrameSizeChanged?.Invoke(Camera, EventArgs.Empty);
                }
                catch
                {
                    rb640.Checked = false;
                }
            }
        }

        private void rb320_CheckedChanged(object sender, EventArgs e)
        {
            if (rb320.Checked)
            {
                try
                {
                    // Comando para invertir horizontalmente la imagen capturada
                    // Command to reverse the captured image horizontally
                    string url = URL + (URL.EndsWith("/") ? "" : "/") + "camera_control.cgi?param=0&value=8";
                    RequestCommand(url);
                    rb640.Checked = false;
                    Resolution = 8;
                    Camera.SetResolution(8);
                    OnFrameSizeChanged?.Invoke(Camera, EventArgs.Empty);
                }
                catch
                {
                    rb320.Checked = false;
                }
            }
        }

        private void BUp_Click(object sender, EventArgs e)
        {
            try
            {
                // Comando para mover la cámara hacia arriba
                // Command to move the camera upwards
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "decoder_control.cgi?command=";
                if (_bMoveUp)
                {
                    // Si la cámara está en movimiento, la paramos
                    // If the camera is in motion, we stop it
                    url += "3";
                }
                else
                {
                    // Si la cámara está parada, la ponemos en movimiento
                    // If the camera is stopped, we put it in motion
                    url += "2";
                }
                RequestCommand(url);
                _bMoveUp = !_bMoveUp;
            }
            catch
            {
            }
        }

        private void BDown_Click(object sender, EventArgs e)
        {
            try
            {
                // Comando para mover la cámara hacia abajo
                // Command to move the camera downwards
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "decoder_control.cgi?command=";
                if (_bMoveDown)
                {
                    // Si la cámara está en movimiento, la paramos
                    // If the camera is in motion, we stop it
                    url += "1";
                }
                else
                {
                    // Si la cámara está parada, la ponemos en movimiento
                    // If the camera is stopped, we put it in motion
                    url += "0";
                }
                RequestCommand(url);
                _bMoveDown = !_bMoveDown;
            }
            catch
            {
            }
        }

        private void BLeft_Click(object sender, EventArgs e)
        {
            try
            {
                // Comando para mover la cámara hacia la izquierda
                // Command to move the camera leftwards
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "decoder_control.cgi?command=";
                if (_bMoveLeft)
                {
                    // Si la cámara está en movimiento, la paramos
                    // If the camera is in motion, we stop it
                    url += "7";
                }
                else
                {
                    // Si la cámara está parada, la ponemos en movimiento
                    // If the camera is stopped, we put it in motion
                    url += "6";
                }
                RequestCommand(url);
                _bMoveLeft = !_bMoveLeft;
            }
            catch
            {
            }
        }

        private void BRight_Click(object sender, EventArgs e)
        {
            try
            {
                // Comando para mover la cámara hacia la derecha
                // Command to move the camera rightwards
                string url = URL + (URL.EndsWith("/") ? "" : "/") + "decoder_control.cgi?command=";
                if (_bMoveRight)
                {
                    // Si la cámara está en movimiento, la paramos
                    // If the camera is in motion, we stop it
                    url += "5";
                }
                else
                {
                    // Si la cámara está parada, la ponemos en movimiento
                    // If the camera is stopped, we put it in motion
                    url += "4";
                }
                RequestCommand(url);
                _bMoveRight = !_bMoveRight;
            }
            catch
            {
            }
        }

        private void CameraConfiguration_Load(object sender, EventArgs e)
        {
            GetCurrentSettings();
        }

        private void CameraConfiguration_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            Hide();
        }
    }
}
