﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;
using System.IO;
using System.Globalization;
using System.Threading.Tasks;
using WatcherCommons.Data;

using Xamarin.Forms;

namespace TWClientApp
{
    public class CameraPage : ContentPage
    {
        // Get client uniqe identifier
        private string _clientId = Guid.NewGuid().ToString("N");
        // Camera data
        private List<CameraInfo> _camList = null;
        // Index of the current selected camera in the _camList List
        private int _selectedCamera = -1;
        // Protocol used to comunicate with the server
        private IStorageManager _storage = null;
        // Main layout 
        private StackLayout _sLayout = null;
        // List of photos layout
        private StackLayout _slPhotos = null;
        // List with the names of the picture files for the current camera
        private List<string> _photos = new List<string>();
        // Image with the current camera frame
        private Image _frame = null;
        public CameraPage()
        {
            Button btn = new Button()
            {
                Text = "Connect",
                HorizontalOptions = LayoutOptions.Center,
                VerticalOptions = LayoutOptions.Center
            };
            btn.Clicked += Btn_Connect;
            Content = btn;
            Device.OnPlatform(() => { Padding = new Thickness(0, 20, 0, 0); });
        }
        private void Sleep()
        {
            DateTime dt = DateTime.Now.AddMilliseconds(1500);
            while (DateTime.Now.CompareTo(dt) < 0) ;
        }
        private async Task TimerCallback()
        {
            while (true)
            {
                try
                {
                    bool wait = true;
                    if ((_frame != null) && (_camList[_selectedCamera].Active))
                    {
                        Stream s = await GetFrame();
                        if (s != null)
                        {
                            ImageSource img = ImageSource.FromStream(() => s);
                            _frame.Source = img;
                            wait = false;
                        }
                    }
                    if (wait)
                    {
                        await Task.Run((Action)Sleep);
                    }
                    await ListPhotos();
                }
                catch
                {
                }
            }
        }
        private async Task<Stream> GetFrame()
        {
            MemoryStream s = new MemoryStream();
            string filename = _camList[_selectedCamera].ID + "_FRAME_" + _clientId + ".jpg";
            if (await _storage.ExistsFile(filename))
            {
                await _storage.DownloadFile(filename, s);
                await _storage.DeleteFile(filename);
                s.Position = 0;
                return s;
            }
            return null;
        }
        private async Task<Stream> GetPhoto(string filename)
        {
            MemoryStream s = new MemoryStream();
            if (await _storage.ExistsFile(filename))
            {
                await _storage.DownloadFile(filename, s);
                s.Position = 0;
                return s;
            }
            return null;
        }
        private async Task<List<CameraInfo>> GetResponse()
        {
            List<CameraInfo> resp = null;
            do
            {
                resp = await _storage.GetResponse(_clientId);
            } while (resp == null);
            return resp;
        }
        private void ConfigButton(Button btn, CameraInfo ci, bool actual)
        {
            if (actual)
            {
                btn.BackgroundColor = ci.Active ? Color.Green : Color.Red;
                btn.TextColor = ci.Active ? Color.Black : Color.White;
                btn.BorderColor = Color.White;
                btn.BorderWidth = 2;
            }
            else
            {
                btn.BackgroundColor = Color.Gray;
                btn.TextColor = Color.Black;
                btn.BorderColor = Color.Black;
                btn.BorderWidth = 1;
            }
        }
        private void InitializePhotos()
        {
            _photos.Clear();
            _slPhotos.Children.Clear();
            _slPhotos.Children.Add(new Label()
            {
                Text = "PHOTOS:",
                HorizontalOptions = LayoutOptions.FillAndExpand,
                BackgroundColor = Color.Yellow,
                TextColor = Color.Black,
                FontSize = 24
            });
        }
        private async Task ListPhotos()
        {
            try
            {
                List<string> lph = await _storage.ListFiles(_camList[_selectedCamera].ID + "_PHOTO_");
                lph.Sort();
                foreach (string ph in lph)
                {
                    if (!_photos.Contains(ph))
                    {
                        Stream s = await GetPhoto(ph);
                        if (s != null)
                        {
                            StackLayout sl = new StackLayout()
                            {
                                Orientation = StackOrientation.Horizontal                                
                            };
                            ImageSource src = ImageSource.FromStream(() => s);
                            Image img = new Image()
                            {
                                HorizontalOptions = LayoutOptions.Start,
                                Source = src,
                                HeightRequest = 100                                
                            };
                            sl.Children.Add(img);
                            StackLayout slh = new StackLayout()
                            {
                                Orientation = StackOrientation.Vertical,
                                HorizontalOptions = LayoutOptions.FillAndExpand
                            };
                            sl.Children.Add(slh);
                            Label lbl = new Label()
                            {
                                Text = DateTime.ParseExact(ph.Substring(ph.IndexOf("_PHOTO_") + 7, 14), "yyyyMMddHHmmss", new DateTimeFormatInfo()).ToString("dd/MM/yyyy HH:mm"),
                                HorizontalOptions = LayoutOptions.FillAndExpand,
                                FontSize = 20,
                                TextColor = Color.Black,
                                BackgroundColor = Color.Silver
                            };
                            slh.Children.Add(lbl);
                            StackLayout sl2 = new StackLayout()
                            {
                                Orientation = StackOrientation.Horizontal,
                                VerticalOptions = LayoutOptions.FillAndExpand,
                                BackgroundColor = Color.White,
                                Padding = new Thickness(10, 10, 10, 10)
                            };
                            slh.Children.Add(sl2);
                            Button btn = new Button()
                            {
                                Text = "Save",
                                HorizontalOptions = LayoutOptions.StartAndExpand,
                                AutomationId = ph,
                                BackgroundColor = Color.Gray,
                                TextColor = Color.Black,
                                BorderColor = Color.Black,
                                BorderWidth = 1
                            };
                            btn.Clicked += Btn_PhotoSave;
                            sl2.Children.Add(btn);
                            btn = new Button()
                            {
                                Text = "Delete",
                                HorizontalOptions = LayoutOptions.EndAndExpand,
                                AutomationId = ph,
                                BackgroundColor = Color.Gray,
                                TextColor = Color.Black,
                                BorderColor = Color.Black,
                                BorderWidth = 1
                            };
                            btn.Clicked += Btn_PhotoDelete;
                            sl2.Children.Add(btn);
                            _slPhotos.Children.Add(sl);
                            _photos.Add(ph);
                        }                        
                    }
                 }
            }
            catch
            {
            }
        }
        private async void Btn_Connect(object sender, EventArgs e)
        {
            _storage = new DropBoxStorage();
            ControlCommand cmd = new ControlCommand();
            cmd.ClientID = _clientId;
            cmd.Command = ControlCommand.cmdGetCameraList;
            await _storage.SendCommand(cmd);
            _camList = await GetResponse();
            _sLayout = new StackLayout();
            foreach (CameraInfo ci in _camList)
            {
                Button btn = new Button()
                {
                    Text = ci.ID,
                    BackgroundColor = Color.Gray,
                    TextColor = Color.Black,
                    BorderColor = Color.Black,
                    BorderWidth = 1,
                    HorizontalOptions = LayoutOptions.FillAndExpand
                };
                btn.Clicked += Btn_Camera;
                _sLayout.Children.Add(btn);
            }
            Content = new ScrollView()
            {
                Content = _sLayout
            };
        }
        private async void Btn_Camera(object sender, EventArgs e)
        {
            for (int ix = 0; ix < _sLayout.Children.Count; ix++)
            {
                if (_sLayout.Children[ix] is Button)
                {
                    if (_sLayout.Children[ix] == sender)
                    {
                        ConfigButton((Button)_sLayout.Children[ix], _camList[ix], true);
                        _selectedCamera = ix;
                    }
                    else
                    {
                        ConfigButton((Button)_sLayout.Children[ix], null, false);
                    }
                }
                else
                {
                    break;
                }
            }
            if (_frame == null)
            {
                _frame = new Image()
                {
                    Aspect = Aspect.AspectFit,
                    HorizontalOptions = LayoutOptions.Center
                };
                _sLayout.Children.Add(_frame);
                StackLayout cambtn = new StackLayout()
                {
                    Orientation = StackOrientation.Horizontal,
                    HorizontalOptions = LayoutOptions.FillAndExpand,
                    BackgroundColor = Color.Black,
                    Padding = new Thickness(0, 4)
                };
                Button btn = new Button()
                {
                    Text = _camList[_selectedCamera].Active ? "Stop" : "Start",
                    HorizontalOptions = LayoutOptions.StartAndExpand,
                    BackgroundColor = Color.Gray,
                    TextColor = Color.Black,
                    BorderColor = Color.Black
                };
                btn.Clicked += Btn_StartStop;
                cambtn.Children.Add(btn);
                btn = new Button()
                {
                    Text = "Photo",
                    HorizontalOptions = LayoutOptions.CenterAndExpand,
                    BackgroundColor = Color.Gray,
                    TextColor = Color.Black,
                    BorderColor = Color.Black
                };
                btn.Clicked += Btn_Photo;
                cambtn.Children.Add(btn);
                btn = new Button()
                {
                    Text = "End Alarm",
                    HorizontalOptions = LayoutOptions.EndAndExpand,
                    BackgroundColor = Color.Gray,
                    TextColor = Color.Black,
                    BorderColor = Color.Black
                };
                btn.Clicked += Btn_EndAlarm;
                cambtn.Children.Add(btn);
                _sLayout.Children.Add(cambtn);
                _slPhotos = new StackLayout();
                _sLayout.Children.Add(_slPhotos);
                InitializePhotos();
                await TimerCallback();
            }
            else
            {
                InitializePhotos();
            }
        }
        private async void Btn_StartStop(object sender, EventArgs e)
        {
            List<CameraInfo> creq = new List<CameraInfo>();
            CameraInfo ci = new CameraInfo()
            {
                Active = !_camList[_selectedCamera].Active,
                ID = _camList[_selectedCamera].ID,
                ClientID = _clientId,
                Photo = false
            };
            creq.Add(ci);
            await _storage.SendRequest(creq);
            creq = await GetResponse();
            (sender as Button).Text = creq[0].Active ? "Stop" : "Start";
            _camList[_selectedCamera] = creq[0];
            for (int ix = 0; ix < _sLayout.Children.Count; ix++)
            {
                if (_sLayout.Children[ix] is Button)
                {
                    if (ix == _selectedCamera)
                    {
                        ConfigButton((Button)_sLayout.Children[ix], _camList[ix], true);
                    }
                    else
                    {
                        ConfigButton((Button)_sLayout.Children[ix], null, false);
                    }
                }
                else
                {
                    break;
                }
            }
        }
        private async void Btn_Photo(object sender, EventArgs e)
        {
            List<CameraInfo> creq = new List<CameraInfo>();
            CameraInfo ci = new CameraInfo()
            {
                Active = _camList[_selectedCamera].Active,
                ID = _camList[_selectedCamera].ID,
                ClientID = _clientId,
                Photo = true
            };
            creq.Add(ci);
            await _storage.SendRequest(creq);
            await GetResponse();
        }
        private async void Btn_EndAlarm(object sender, EventArgs e)
        {
            ControlCommand cmd = new ControlCommand();
            cmd.ClientID = _clientId;
            cmd.Command = ControlCommand.cmdStopAlarm;
            await _storage.SendCommand(cmd);
            await GetResponse();
            Button btn = new Button()
            {
                Text = "Connect",
                HorizontalOptions = LayoutOptions.Center,
                VerticalOptions = LayoutOptions.Center
            };
            btn.Clicked += Btn_Connect;
            Content = btn;
            _camList = null;
            _selectedCamera = -1;
            _storage = null;
            _sLayout = null;
            _frame = null;
        }
        private async void Btn_PhotoSave(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                string photo = ((Button)sender).AutomationId;
                int ix = _photos.IndexOf(photo);
                if (ix >= 0)
                {
                    MemoryStream s = await GetPhoto(photo) as MemoryStream;
                    byte[] data = s.ToArray();
                    await DependencyService.Get<IImageManager>().SaveImage(photo, data);
                    await DisplayAlert("Image Saved", "The image has been saved", "OK");
                }
            }            
        }
        private async void Btn_PhotoDelete(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                string photo = ((Button)sender).AutomationId;
                await _storage.DeleteFile(photo);
                int ix = _photos.IndexOf(photo);
                if (ix >= 0)
                {
                    _slPhotos.Children.RemoveAt(ix + 1);
                    _photos.Remove(photo);
                }
            }
        }
    }
}
