﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using Dropbox.Api;
using Dropbox.Api.Stone;
using Dropbox.Api.Files;
using WatcherCommons.Data;

namespace TWClientApp
{
    public class DropBoxStorage : IStorageManager
    {
        private const string _accessKey = "";
        public DropBoxStorage()
        {            
        }
        public async Task DownloadFile(string filename, Stream s)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                IDownloadResponse<FileMetadata> resp = await client.Files.DownloadAsync("/" + filename);
                Stream ds = await resp.GetContentAsStreamAsync();
                await ds.CopyToAsync(s);
                ds.Dispose();
            }
        }
        public async Task DeleteFile(string filename)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                await client.Files.DeleteAsync("/" + filename);
            }
        }
        public async Task<bool> ExistsFile(string filename)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                try
                {
                    bool more = true;
                    var list = await client.Files.ListFolderAsync("");
                    while (more)
                    {
                        foreach (var item in list.Entries.Where(i => i.IsFile))
                        {
                            if (item.Name.StartsWith(filename))
                            {
                                return true;
                            }
                        }
                        more = list.HasMore;
                        if (more)
                        {
                            list = await client.Files.ListFolderContinueAsync(list.Cursor);
                        }
                    }
                }
                catch
                {
                }
            }
            return false;
        }
        public async Task<List<string>> ListFiles(string model)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                var list = await client.Files.ListFolderAsync("");
                bool more = true;
                List<string> files = new List<string>();
                while (more)
                {                    
                    foreach (var item in list.Entries.Where(i => i.IsFile))
                    {
                        if (item.Name.StartsWith(model))
                        {
                            files.Add(item.Name);
                        }
                    }
                    more = list.HasMore;
                    if (more)
                    {
                        list = await client.Files.ListFolderContinueAsync(list.Cursor);
                    }
                }
                return files;
            }
        }
        public async Task SendCommand(ControlCommand cmd)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    ControlCommand.ToJSON(ms, cmd);
                    ms.Position = 0;
                    string filename = "cmd_" + cmd.ClientID + ".json";
                    while (await ExistsFile(filename)) ;
                    await client.Files.UploadAsync("/" + filename, null, false, null, false, ms);
                }
            }
        }
        public async Task SendRequest(List<CameraInfo> req)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    CameraInfo.ToJSON(ms, req);
                    ms.Position = 0;
                    string filename = "req_" + req[0].ClientID + ".json";
                    while (await ExistsFile(filename)) ;
                    await client.Files.UploadAsync("/" + filename, null, false, null, false, ms);
                }
            }            
        }
        public async Task<List<CameraInfo>> GetResponse(string id)
        {
            string filename = "resp_" + id + ".json";
            while (!await ExistsFile(filename)) ;
            using (MemoryStream ms = new MemoryStream())
            {
                try
                {
                    await DownloadFile(filename, ms);
                    ms.Position = 0;
                    List<CameraInfo> lci = CameraInfo.FromJSON(ms);
                    if ((lci != null) && (lci.Count > 0))
                    {
                        await ClearFiles(id);
                        return lci;
                    }
                }
                catch
                {
                    return null;
                }
            }
            return null;
        }
        private async Task ClearFiles(string id)
        {
            using (DropboxClient client = new DropboxClient(_accessKey))
            {
                try
                {
                    await client.Files.DeleteAsync("/resp_" + id + ".json");
                }
                catch
                {
                }
                try
                {
                    await client.Files.DeleteAsync("/req_" + id + ".json");
                }
                catch
                {
                }
                try
                {
                    await client.Files.DeleteAsync("/cmd_" + id + ".json");
                }
                catch
                {
                }
            }
        }
    }
}
