﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Configuration;
using ThiefWatcher.Properties;

namespace ThiefWatcher.Configuration
{
    public static class ConfigurationHelper
    {
        public static void SaveConnectionString(string name, string cs, bool save)
        {
            System.Configuration.Configuration con = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            if (con != null)
            {
                ConnectionStringsSection cons = con.ConnectionStrings;
                if (cons.ConnectionStrings[name] == null)
                {
                    cons.ConnectionStrings.Add(new ConnectionStringSettings(name, cs));
                }
                else
                {
                    if (string.IsNullOrEmpty(cs))
                    {
                        cons.ConnectionStrings.Remove(cons.ConnectionStrings[name]);
                    }
                    else
                    {
                        cons.ConnectionStrings[name].ConnectionString = cs;
                    }
                }
                if (save)
                {
                    con.Save(ConfigurationSaveMode.Modified);
                    ConfigurationManager.RefreshSection("connectionStrings");
                }
            }
        }
        public static void SaveConnectionString(string name, string cs, System.Configuration.Configuration con)
        {
            ConnectionStringsSection cons = con.ConnectionStrings;
            if (cons.ConnectionStrings[name] == null)
            {
                cons.ConnectionStrings.Add(new ConnectionStringSettings(name, cs));
            }
            else
            {
                if (string.IsNullOrEmpty(cs))
                {
                    cons.ConnectionStrings.Remove(cons.ConnectionStrings[name]);
                }
                else
                {
                    cons.ConnectionStrings[name].ConnectionString = cs;
                }
            }
        }
        public static void SaveParameter(AppSettingsSection sec, string param, string value)
        {
            if (string.IsNullOrEmpty(value))
            {
                if (sec.Settings[param] != null)
                {
                    sec.Settings[param].Value = null;
                }
            }
            else
            {
                if (sec.Settings[param] == null)
                {
                    sec.Settings.Add(param, value);
                }
                else
                {
                    sec.Settings[param].Value = value;
                }
            }
        }
        public static string GetProtocol(Type type)
        {
            ProtocolsSection protocols = (ProtocolsSection)ConfigurationManager.GetSection(Resources.STR_PROTOCOLSECTION);
            foreach (ProtocolData pd in protocols.Protocols)
            {
                if (Type.GetType(pd.ProtocolType).Equals(type))
                {
                    return pd.Name;
                }
            }
            return "";
        }
    }
}
