﻿using System;
using System.Collections.Generic;
using System.Configuration;

namespace ThiefWatcher.Configuration
{
    public class ProtocolData : ConfigurationElement
    {
        [ConfigurationProperty("name", IsRequired = true)]
        public string Name
        {
            get
            {
                return (string)(this["name"]);
            }
            set
            {
                this["name"] = value;
            }
        }
        [ConfigurationProperty("class", IsRequired = true)]
        public string Class
        {
            get
            {
                return (string)(this["class"]);
            }
            set
            {
                this["class"] = value;
            }
        }
        [ConfigurationProperty("type", IsRequired = true)]
        public string ProtocolType
        {
            get
            {
                return (string)(this["type"]);
            }
            set
            {
                this["type"] = value;
            }
        }
    }
    [ConfigurationCollection(typeof(ProtocolData))]
    public class ProtocolCollection : ConfigurationElementCollection
    {
        public void Add(ProtocolData data)
        {
            BaseAdd(data);
        }
        public override ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.BasicMapAlternate;
            }
        }
        protected override string ElementName
        {
            get
            {
                return "protocolData";
            }
        }
        protected override bool IsElementName(string elementName)
        {
            return elementName.Equals("protocolData", StringComparison.InvariantCultureIgnoreCase);
        }

        public override bool IsReadOnly()
        {
            return false;
        }

        protected override ConfigurationElement CreateNewElement()
        {
            return new ProtocolData();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((ProtocolData)(element)).Name;
        }

        public ProtocolData this[int idx]
        {
            get { return (ProtocolData)BaseGet(idx); }
        }
        public void RemoveAt(int ix)
        {
            BaseRemoveAt(ix);
        }
        public IEnumerable<ProtocolData> GetProtocols(string cls)
        {
            foreach (ProtocolData pd in this)
            {
                if (string.Compare(pd.Class, cls, true) == 0)
                {
                    yield return pd;
                }
            }
        }
        public ProtocolData GetByName(string name)
        {
            foreach (ProtocolData pd in this)
            {
                if (string.Compare(pd.Name, name, true) == 0)
                {
                    return pd;
                }
            }
            return null;
        }
    }
    public class ProtocolsSection : ConfigurationSection
    {
        [ConfigurationProperty("protocols")]
        public ProtocolCollection Protocols
        {
            get
            {
                return (ProtocolCollection)(base["protocols"]);
            }
            set
            {
                base["protocols"] = value;
            }
        }
        public void RemoveAt(int ix)
        {
            Protocols.RemoveAt(ix);
        }
    }
}
