﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Configuration;
using System.Windows.Forms;
using System.Threading.Tasks;
using WatcherCommons.Interfaces;
using WatcherCommons.Data;
using ThiefWatcher.Properties;
using ThiefWatcher.Configuration;

namespace ThiefWatcher
{
    public partial class ControlPanel : Form
    {
        private List<frmCamera> _cameras = new List<frmCamera>();
        private CameraMode _mode = CameraMode.Configuration;
        private DateTime? _dtStart = null;
        private DateTime? _dtEnd = null;
        private ITrigger _trigger = null;
        private IAlarmChannel _alarm = null;
        private IStorageManager _storage = null;

        public ControlPanel()
        {
            InitializeComponent();
            Text = UIResources.TTL_CTLPANEL;
            gbComm.Text = UIResources.GB_COMM;
            lTriggerPc.Text = UIResources.LAB_TRIGGERPC;
            lTriggerConStr.Text = UIResources.LAB_CONSTRING;
            lTake.Text = UIResources.LAB_TAKE;
            lPhotos.Text = UIResources.LAB_PHOTOS;
            lSeconds.Text = UIResources.LAB_SECS;
            lStartDate.Text = UIResources.LAB_STARTD;
            dtpStart.CustomFormat = UIResources.FMT_DATE;
            lEndDate.Text = UIResources.LAB_ENDD;
            dtpEnd.CustomFormat = UIResources.FMT_DATE;
            gbAlarm.Text = UIResources.GB_NOTIFICATIONS;
            lProtocol.Text = UIResources.LAB_MSGPROTOCOL;
            bTestAlarm.Text = UIResources.BTN_ALARM;
            lConnString.Text = UIResources.LAB_CONSTRING;
            lMessage.Text = UIResources.LAB_MESSAGE;
            gbStorage.Text = UIResources.GB_STORAGE;
            lStProtocol.Text = UIResources.LAB_STPROTOCOL;
            lStConnStr.Text = UIResources.LAB_CONSTRING;
            lCont.Text = UIResources.LAB_STCONT;
            gbButtons.Text = UIResources.GB_CONTROLS;
            bSimulacrum.Text = UIResources.BTN_TEST;
            bStart.Text = UIResources.BTN_START;
            bSave.Text = UIResources.BTN_SAVE;
            ReadConfiguration();
            AllowClose = false;
        }
        public bool AllowClose { get; set; }
        public void AddCamera(frmCamera camform)
        {
            _cameras.Add(camform);
        }
        public void RemoveCamera(frmCamera camform)
        {
            _cameras.Remove(camform);
        }
        public void ReadConfiguration()
        {
            ProtocolsSection protocols = (ProtocolsSection)ConfigurationManager.GetSection(Resources.STR_PROTOCOLSECTION);
            int ix = 0;
            int ixs = -1;
            string ap = ConfigurationManager.AppSettings[Resources.PARAM_TRIGGERP];
            cbTriggerPc.Items.Clear();
            foreach (ProtocolData pd in protocols.Protocols.GetProtocols(Resources.STR_TRIGGERCLASS))
            {
                cbTriggerPc.Items.Add((ITrigger)Type.GetType(pd.ProtocolType).GetConstructor(Type.EmptyTypes).Invoke(null));
                if (string.Compare(pd.Name, ap, true) == 0)
                {
                    ixs = ix;
                }
                ix++;
            }
            cbTriggerPc.SelectedIndex = ixs;
            if (ConfigurationManager.ConnectionStrings[Resources.PARAM_TRIGGERCON] != null)
            {
                txtTriggerConStr.Text = ConfigurationManager.ConnectionStrings[Resources.PARAM_TRIGGERCON].ConnectionString;
            }
            else
            {
                txtTriggerConStr.Text = "";
            }
            txtTake.Text = ConfigurationManager.AppSettings[Resources.PARAM_NPHOTOS];
            txtSecs.Text = ConfigurationManager.AppSettings[Resources.PARAM_PHOTOFREQ];
            if (string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.PARAM_STARTDT]))
            {
                dtpStart.Checked = false;
                dtpStart.Value = DateTime.Now;
                _dtStart = null;
            }
            else
            {
                dtpStart.Checked = true;
                dtpStart.Value = DateTime.ParseExact(ConfigurationManager.AppSettings[Resources.PARAM_STARTDT], UIResources.FMT_DATE, Thread.CurrentThread.CurrentCulture.DateTimeFormat);
                _dtStart = dtpStart.Value;
            }
            if (string.IsNullOrEmpty(ConfigurationManager.AppSettings[Resources.PARAM_ENDDT]))
            {
                dtpEnd.Checked = false;
                dtpEnd.Value = DateTime.Now;
                _dtEnd = null;
            }
            else
            {
                dtpEnd.Checked = true;
                dtpEnd.Value = DateTime.ParseExact(ConfigurationManager.AppSettings[Resources.PARAM_ENDDT], UIResources.FMT_DATE, Thread.CurrentThread.CurrentCulture.DateTimeFormat);
                _dtEnd = dtpEnd.Value;
            }            
            ix = 0;
            ixs = -1;
            ap = ConfigurationManager.AppSettings[Resources.PARAM_ALARMP];
            cbProtocol.Items.Clear();
            foreach (ProtocolData pd in protocols.Protocols.GetProtocols(Resources.STR_ALARMCLASS))
            {
                cbProtocol.Items.Add((IAlarmChannel)Type.GetType(pd.ProtocolType).GetConstructor(Type.EmptyTypes).Invoke(null));
                if (string.Compare(pd.Name, ap, true) == 0)
                {
                    ixs = ix;
                }
                ix++;
            }
            cbProtocol.SelectedIndex = ixs;
            if (ConfigurationManager.ConnectionStrings[Resources.PARAM_ALARMCON] != null)
            {
                txtConnString.Text = ConfigurationManager.ConnectionStrings[Resources.PARAM_ALARMCON].ConnectionString;
            }
            else
            {
                txtConnString.Text = "";
            }
            txtMessage.Text = ConfigurationManager.AppSettings[Resources.PARAM_ALARMMSG];
            ix = 0;
            ixs = -1;
            ap = ConfigurationManager.AppSettings[Resources.PARAM_STPROTOCOL];
            cbStProtocol.Items.Clear();
            foreach (ProtocolData pd in protocols.Protocols.GetProtocols(Resources.STR_STORAGECLASS))
            {
                cbStProtocol.Items.Add((IStorageManager)Type.GetType(pd.ProtocolType).GetConstructor(Type.EmptyTypes).Invoke(null));
                if (string.Compare(pd.Name, ap, true) == 0)
                {
                    ixs = ix;
                }
                ix++;
            }
            cbStProtocol.SelectedIndex = ixs;
            if (ConfigurationManager.ConnectionStrings[Resources.PARAM_STCONNECTION] != null)
            {
                txtStConnstr.Text = ConfigurationManager.ConnectionStrings[Resources.PARAM_STCONNECTION].ConnectionString;
            }
            else
            {
                txtStConnstr.Text = "";
            }
            txtCont.Text = ConfigurationManager.AppSettings[Resources.PARAM_CONTPATH];
        }
        private void TriggerFired(object sender, EventArgs e)
        {
            try
            {
                foreach (frmCamera cam in _cameras)
                {
                    BeginInvoke((Action)delegate () { cam.Mode = CameraMode.Surveillance; });
                }
                _alarm.SendAlarm();
                _trigger.Stop();
            }
            catch
            {
            }
        }
        private void ControlPanel_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !AllowClose;
        }

        private void bSave_Click(object sender, EventArgs e)
        {
            System.Configuration.Configuration con = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            if (con != null)
            {
                try
                {
                    AppSettingsSection sec = con.AppSettings;
                    ConfigurationHelper.SaveConnectionString(Resources.PARAM_TRIGGERCON, txtTriggerConStr.Text, con);
                    if (cbTriggerPc.SelectedItem != null)
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_TRIGGERP, ConfigurationHelper.GetProtocol(cbTriggerPc.SelectedItem.GetType()));
                    }
                    else
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_TRIGGERP, null);
                    }
                    ConfigurationHelper.SaveParameter(sec, Resources.PARAM_NPHOTOS, null);
                    if (string.IsNullOrEmpty(txtTake.Text))
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_NPHOTOS, null);
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_PHOTOFREQ, null);
                    }
                    else
                    {
                        int test;
                        if (int.TryParse(txtTake.Text, out test))
                        {
                            if (int.TryParse(txtSecs.Text, out test))
                            {
                                ConfigurationHelper.SaveParameter(sec, Resources.PARAM_NPHOTOS, txtTake.Text);
                                ConfigurationHelper.SaveParameter(sec, Resources.PARAM_PHOTOFREQ, txtSecs.Text);
                            }
                            else
                            {
                                txtSecs.SelectAll();
                                txtSecs.Focus();
                                throw new Exception(UIResources.ERR_BADNUMBER);
                            }
                        }
                        else
                        {
                            txtTake.SelectAll();
                            txtTake.Focus();
                            throw new Exception(UIResources.ERR_BADNUMBER);
                        }
                    }
                    if (dtpStart.Checked)
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_STARTDT, dtpStart.Value.ToString(UIResources.FMT_DATE));
                    }
                    else
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_STARTDT, null);
                    }
                    if (dtpEnd.Checked)
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_ENDDT, dtpEnd.Value.ToString(UIResources.FMT_DATE));
                    }
                    else
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_ENDDT, null);
                    }
                    if (cbProtocol.SelectedItem != null)
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_ALARMP, ConfigurationHelper.GetProtocol(cbProtocol.SelectedItem.GetType()));
                    }
                    else
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_ALARMP, null);
                    }
                    ConfigurationHelper.SaveParameter(sec, Resources.PARAM_ALARMMSG, txtMessage.Text);
                    ConfigurationHelper.SaveConnectionString(Resources.PARAM_ALARMCON, txtConnString.Text, con);
                    if (cbStProtocol.SelectedItem != null)
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_STPROTOCOL, ConfigurationHelper.GetProtocol(cbStProtocol.SelectedItem.GetType()));
                    }
                    else
                    {
                        ConfigurationHelper.SaveParameter(sec, Resources.PARAM_STPROTOCOL, null);
                    }
                    ConfigurationHelper.SaveParameter(sec, Resources.PARAM_CONTPATH, txtCont.Text);
                    ConfigurationHelper.SaveConnectionString(Resources.PARAM_STCONNECTION, txtStConnstr.Text, con);
                    con.Save(ConfigurationSaveMode.Modified);
                    ConfigurationManager.RefreshSection("appSettings");
                    ConfigurationManager.RefreshSection("connectionStrings");
                    MessageBox.Show(UIResources.MSG_CTLCFGSAVED);   
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }
        private void bSimulacrum_Click(object sender, EventArgs e)
        {
            try
            {
                if (_mode == CameraMode.Configuration)
                {
                    _storage = cbStProtocol.SelectedItem as IStorageManager;
                    if (_storage != null)
                    {
                        _storage.ConnsecionString = txtStConnstr.Text;
                        _storage.ContainerPath = txtCont.Text;
                        foreach (frmCamera cam in _cameras)
                        {
                            cam.StorageManager = _storage;
                            cam.Mode = CameraMode.Simulacrum;
                        }
                        _mode = CameraMode.Simulacrum;
                        bSimulacrum.Text = UIResources.BTN_ENDTEST;
                        bStart.Enabled = false;
                        bgTrigger.RunWorkerAsync();
                    }
                }
                else
                {                    
                    bgTrigger.CancelAsync();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bStart_Click(object sender, EventArgs e)
        {
            try
            {
                if (_mode == CameraMode.Configuration)
                {
                    _trigger = cbTriggerPc.SelectedItem as ITrigger;
                    _alarm = cbProtocol.SelectedItem as IAlarmChannel;
                    _storage = cbStProtocol.SelectedItem as IStorageManager;
                    if ((_trigger != null) && (_alarm != null) && (_storage != null))
                    {
                        _trigger.Initialize();
                        _trigger.ConnectionString = txtTriggerConStr.Text;
                        _trigger.OnTriggerFired += new EventHandler(TriggerFired);
                        if (dtpStart.Checked)
                        {
                            _dtStart = dtpStart.Value;
                        }
                        else
                        {
                            _dtStart = null;
                        }
                        if (dtpEnd.Checked)
                        {
                            _dtEnd = dtpEnd.Value;
                        }
                        else
                        {
                            _dtEnd = null;
                        }
                        _storage.ConnsecionString = txtStConnstr.Text;
                        _storage.ContainerPath = txtCont.Text;
                        foreach (frmCamera cam in _cameras)
                        {
                            cam.StorageManager = _storage;
                            if (string.IsNullOrEmpty(txtTake.Text) || string.IsNullOrEmpty(txtSecs.Text))
                            {
                                cam.PhotoCount = -1;
                            }
                            else
                            {
                                cam.PhotoCount = int.Parse(txtTake.Text);
                                cam.PhotoTS = new TimeSpan(0, 0, int.Parse(txtSecs.Text));
                            }
                        }
                        _alarm.Initialice();
                        _alarm.ConnectionString = txtConnString.Text;
                        _alarm.MessageText = txtMessage.Text;                        
                        bSimulacrum.Enabled = false;
                        bStart.Text = UIResources.BTN_STOP;
                        _mode = CameraMode.Surveillance;
                        bgTrigger.RunWorkerAsync();
                    }
                    else
                    {
                        MessageBox.Show(UIResources.ERR_NOTALLPROTOCOLS);
                    }
                }
                else
                {
                    if (_trigger != null)
                    {
                        _trigger.Initialize();
                    }
                    bgTrigger.CancelAsync();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bgTrigger_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            while ((_mode != CameraMode.Simulacrum) && (_dtStart != null) && (DateTime.Now.CompareTo(_dtStart) < 0))
            {
                if (bgTrigger.CancellationPending)
                {
                    return;
                }
            }
            if (_mode != CameraMode.Simulacrum)
            {
                _trigger.Start();
            }
            while ((_mode == CameraMode.Simulacrum) || (_dtEnd == null) || (DateTime.Now.CompareTo(_dtEnd) < 0))
            {
                try
                {
                    if (bgTrigger.CancellationPending)
                    {
                        return;
                    }
                    foreach (ControlCommand cc in _storage.GetCommands())
                    {
                        switch (cc.Command)
                        {
                            case ControlCommand.cmdGetCameraList:
                                List<CameraInfo> lci = new List<CameraInfo>();
                                foreach (frmCamera fc in _cameras)
                                {
                                    CameraInfo ci = new CameraInfo();
                                    ci.ID = fc.ID;
                                    ci.Active = fc.Active;
                                    ci.Width = fc.Camera.FrameSize.Width;
                                    ci.Height = fc.Camera.FrameSize.Height;
                                    ci.ClientID = cc.ClientID;
                                    lci.Add(ci);
                                    BeginInvoke((Action)delegate { fc.AddClient(cc.ClientID); });
                                }
                                _storage.SendResponse(lci);
                                break;
                            case ControlCommand.cmdStopAlarm:
                                foreach (frmCamera cam in _cameras)
                                {
                                    BeginInvoke((Action)delegate { cam.Mode = CameraMode.Configuration; });
                                    BeginInvoke((Action)delegate { cam.Stop(); });
                                    BeginInvoke((Action)delegate { cam.ClearClients(); });
                                }
                                if (_mode != CameraMode.Simulacrum)
                                {
                                    _trigger.Start();
                                }
                                lci = new List<CameraInfo>();
                                lci.Add(new CameraInfo()
                                {
                                    ClientID = cc.ClientID
                                });
                                _storage.SendResponse(lci);
                                break;
                        }
                    }
                    foreach (List<CameraInfo> lci in _storage.GetRequests())
                    {
                        List<CameraInfo> lcir = new List<CameraInfo>();
                        foreach (CameraInfo ci in lci)
                        {
                            foreach (frmCamera cam in _cameras)
                            {
                                CameraInfo cir = null;
                                IAsyncResult iar = BeginInvoke((Action)delegate () { cir = cam.CameraRequest(ci, _mode); });
                                while (!iar.IsCompleted) ;
                                if (cir != null)
                                {
                                    lcir.Add(cir);
                                }
                            }
                        }
                        if (lcir.Count > 0)
                        {
                            _storage.SendResponse(lcir);
                        }
                    }
                }
                catch (Exception ex)
                {
                    string s = ex.Message;
                }
            }
        }

        private void bgTrigger_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                foreach (frmCamera cam in _cameras)
                {
                    cam.Mode = CameraMode.Configuration;
                    cam.Stop();
                    cam.ClearClients();
                }                
                if (_mode == CameraMode.Simulacrum)
                {
                    bSimulacrum.Text = UIResources.BTN_TEST;
                    bStart.Enabled = true;
                }
                else
                {
                    bSimulacrum.Enabled = true;
                    bStart.Text = UIResources.BTN_START;
                }
                _mode = CameraMode.Configuration;
            }
            else
            {
                bgTrigger.RunWorkerAsync();
            }
        }

        private void bTestAlarm_Click(object sender, EventArgs e)
        {
            try
            {
                _alarm = cbProtocol.SelectedItem as IAlarmChannel;
                if (_alarm != null)
                {
                    _alarm.Initialice();
                    _alarm.ConnectionString = txtConnString.Text;
                    _alarm.MessageText = txtMessage.Text;
                    _alarm.SendAlarm();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
    }
}
