﻿using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Configuration;
using System.Windows.Forms;
using ThiefWatcher.Properties;
using ThiefWatcher.Configuration;

namespace ThiefWatcher
{
    public partial class Watcher : Form
    {
        private ControlPanel _ctlPanel = null;
        public Watcher()
        {            
            InitializeComponent();
            Text = UIResources.TTL_WATCHER;
            mnuFile.Text = UIResources.MNU_FILE;
            opNew.Text = UIResources.OPT_NEW;
            opProtocols.Text = UIResources.OPT_PROTOCOLS;
            opExit.Text = UIResources.OPT_EXIT;
            mnuWindow.Text = UIResources.MNU_WINDOW;
            mnuHelp.Text = UIResources.MNU_HELP;
        }
        private void RemoveCamera(object sender, FormClosedEventArgs e)
        {
            _ctlPanel.RemoveCamera(sender as frmCamera);
        }
        private void AddProtocol(Type t, string pclass, ProtocolsSection protocols)
        {
            object obj = t.GetConstructor(Type.EmptyTypes).Invoke(null);
            string id = obj.ToString();
            ProtocolData pd = protocols.Protocols.GetByName(id);
            if (pd != null)
            {
                throw new Exception(string.Format(UIResources.ERR_DUPLICATEPID, id));
            }
            pd = new ProtocolData();
            pd.Class = pclass;
            pd.Name = id;
            pd.ProtocolType = t.AssemblyQualifiedName;
            protocols.Protocols.Add(pd);
        }
        private void opNew_Click(object sender, EventArgs e)
        {
            try
            {
                SelectCamera dlg = new SelectCamera();
                if (dlg.ShowDialog(this) == DialogResult.OK)
                {
                    if (dlg.Camera != null)
                    {
                        frmCamera frm = new frmCamera();
                        frm.MdiParent = this;                        
                        frm.Show();
                        frm.Camera = dlg.Camera;
                        frm.FormClosed += new FormClosedEventHandler(RemoveCamera);
                        _ctlPanel.AddCamera(frm);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void Watcher_Load(object sender, EventArgs e)
        {
            try
            {
                _ctlPanel = new ControlPanel();
                _ctlPanel.MdiParent = this;                
                CamerasSection cameras = (CamerasSection)ConfigurationManager.GetSection(Resources.STR_CAMSECTION);
                foreach (CameraData cam in cameras.Cameras)
                {
                    frmCamera frm = new frmCamera(cam);
                    frm.MdiParent = this;
                    frm.Show();
                    frm.FormClosed += new FormClosedEventHandler(RemoveCamera);
                    _ctlPanel.AddCamera(frm);
                }
                _ctlPanel.Show();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void opExit_Click(object sender, EventArgs e)
        {
            _ctlPanel.AllowClose = true;
            Application.Exit();
        }

        private void opProtocols_Click(object sender, EventArgs e)
        {
            ofDlg.Filter = UIResources.FIL_PROTOCOLS;
            ofDlg.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath);
            if (ofDlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    System.Configuration.Configuration con = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                    ProtocolsSection protocols = (ProtocolsSection)con.GetSection(Resources.STR_PROTOCOLSECTION);
                    Assembly asm = Assembly.LoadFrom(ofDlg.FileName);
                    int pnum = 0;
                    foreach (Type t in asm.GetTypes())
                    {
                        if (!t.IsAbstract)
                        {                            
                            if (t.GetInterface("IAlarmChannel") != null)
                            {
                                AddProtocol(t, Resources.STR_ALARMCLASS, protocols);
                                pnum++;
                            }
                            else if (t.GetInterface("IStorageManager") != null)
                            {
                                AddProtocol(t, Resources.STR_STORAGECLASS, protocols);
                                pnum++;
                            }
                            else if (t.GetInterface("ITrigger") != null)
                            {
                                AddProtocol(t, Resources.STR_TRIGGERCLASS, protocols);
                                pnum++;
                            }
                            else if (t.GetInterface("IWatcherCamera") != null)
                            {
                                AddProtocol(t, Resources.STR_CAMERACLASS, protocols);
                                pnum++;
                            }
                        }
                    }
                    con.Save(ConfigurationSaveMode.Modified);
                    ConfigurationManager.RefreshSection(Resources.STR_PROTOCOLSECTION);
                    if (pnum > 0)
                    {
                        _ctlPanel.ReadConfiguration();
                    }
                    MessageBox.Show(string.Format(UIResources.MSG_PROTOCOLS, pnum.ToString()));
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuHelp_Click(object sender, EventArgs e)
        {
            try
            {
                Process p = new Process();
                p.StartInfo.FileName = UIResources.URI_HELP;
                p.Start();
            }
            catch
            {
            }
        }
    }
}
