﻿using System.Windows.Forms;
using System.Drawing;
using VAPIXProtocol.Properties;

namespace VAPIXProtocol.Commands
{
    public class CmdAreaZoom : CmdBase
    {
        private const string cX = "udX";
        private const string cY = "udY";
        private const string cZ = "udZ";
        private const string cW = "udW";
        private const string cH = "udH";
        private int _width = 0;
        private int _height = 0;

        public CmdAreaZoom(string p, TreeView tvparams)
            : base(p, tvparams)
        {

        }
        public override string GetCommandUri(Control container) 
        {
            int x = _width / 2;
            int y = _height / 2 ;
            int z = 100;
            int w = _width;
            int h = _height;
            foreach (Control c in container.Controls)
            {
                NumericUpDown ud = c as NumericUpDown;
                if (ud != null)
                {
                    switch (ud.Name)
                    {
                        case cX:
                            x = (int)ud.Value;
                            break;
                        case cY:
                            y = (int)ud.Value;
                            break;
                        case cZ:
                            z = (int)ud.Value;
                            break;
                        case cW:
                            w = (int)ud.Value;
                            break;
                        case cH:
                            h = (int)ud.Value;
                            break;
                    }
                }
            }
            return "axis-cgi/com/ptz.cgi?" + Command + "=" + x.ToString() + "," + y.ToString() + "," + z.ToString() + "&imagewidth=" + w.ToString() + "&imageheight=" + h.ToString();
        }
        public override void CreateUI(Control container)
        {
            base.CreateUI(container);
            Label lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = "X:";
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = 16;
            lbl.Top = (container.Height - lbl.Height) / 4;
            NumericUpDown ud = new NumericUpDown();
            ud.Name = cX;
            ud.Minimum = 0;
            ud.Maximum = (decimal)_width;
            ud.Value = (decimal)_width / 2;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (container.Height - ud.Height) / 4;
            ud.Width = 40;
            lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = "Y:";
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = ud.Right + 16;
            lbl.Top = (container.Height - lbl.Height) / 4;
            ud = new NumericUpDown();
            ud.Name = cY;
            ud.Minimum = 0;
            ud.Maximum = (decimal)_height;
            ud.Value = (decimal)_height / 2;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (container.Height - ud.Height) / 4;
            ud.Width = 40;
            lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = UIResources.LAB_ZOOM;
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = ud.Right + 16;
            lbl.Top = (container.Height - lbl.Height) / 4;
            ud = new NumericUpDown();
            ud.Name = cZ;
            ud.Minimum = 0;
            ud.Maximum = (decimal)int.MaxValue;
            ud.Value = 100;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (container.Height - ud.Height) / 4;
            ud.Width = 40;
            lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = UIResources.LAB_WIDTH;
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = 16;
            lbl.Top = (3 * (container.Height - lbl.Height)) / 4;
            ud = new NumericUpDown();
            ud.Name = cX;
            ud.Minimum = 0;
            ud.Maximum = (decimal)_width;
            ud.Value = (decimal)_width;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (3 * (container.Height - ud.Height)) / 4;
            ud.Width = 40;
            lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = UIResources.LAB_HEIGHT;
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = ud.Right + 16;
            lbl.Top = (3 * (container.Height - lbl.Height)) / 4;
            ud = new NumericUpDown();
            ud.Name = cY;
            ud.Minimum = 0;
            ud.Maximum = (decimal)_height;
            ud.Value = (decimal)_height;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (3 * (container.Height - ud.Height)) / 4;
            ud.Width = 40;            
        }
        protected override void ReadParams()
        {
            string[] reso = GetParamValue("Image.I0.Appearance.Resolution").Split('x');
            int.TryParse(reso[0], out _width);
            int.TryParse(reso[1], out _height);
        }
    }
}
