﻿using System;
using System.Windows.Forms;
using VAPIXProtocol.Properties;

namespace VAPIXProtocol.Commands
{
    public abstract class CmdBase
    {
        protected TreeView _tvparams = null;

        protected CmdBase(string p, TreeView tvparams)
        {
            _tvparams = tvparams;
        }
        public static CmdBase CreateCommand(string cmd, TreeView tvparams)
        {
            string[] cmdp = cmd.Split('=');            
            switch (cmdp[0].ToLower())
            {
                case "center":
                    return new CmdCenter(cmdp[1], tvparams) { Command = cmdp[0] };
                case "areazoom":
                    return new CmdAreaZoom(cmdp[1], tvparams) { Command = cmdp[0] };
                case "move":
                    return new CmdMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "pan":
                    return new CmdPan(cmdp[1], tvparams) { Command = cmdp[0] };
                case "tilt":
                    return new CmdTilt(cmdp[1], tvparams) { Command = cmdp[0] };
                case "zoom":
                    return new CmdZoom(cmdp[1], tvparams) { Command = cmdp[0] };
                case "focus":
                    return new CmdFocus(cmdp[1], tvparams) { Command = cmdp[0] };
                case "iris":
                    return new CmdIris(cmdp[1], tvparams) { Command = cmdp[0] };
                case "brightness":
                    return new CmdBrightness(cmdp[1], tvparams) { Command = cmdp[0] };
                case "rpan":
                    return new CmdRPan(cmdp[1], tvparams) { Command = cmdp[0] };
                case "rtilt":
                    return new CmdRTilt(cmdp[1], tvparams) { Command = cmdp[0] };
                case "rzoom":
                    return new CmdRZoom(cmdp[1], tvparams) { Command = cmdp[0] };
                case "rfocus":
                    return new CmdRFocus(cmdp[1], tvparams) { Command = cmdp[0] };
                case "riris":
                    return new CmdRIris(cmdp[1], tvparams) { Command = cmdp[0] };
                case "rbrightness":
                    return new CmdRBrightness(cmdp[1], tvparams) { Command = cmdp[0] };
                case "autofocus":
                    return new CmdAutofocus(cmdp[1], tvparams) { Command = cmdp[0] };
                case "autoiris":
                    return new CmdAutoiris(cmdp[1], tvparams) { Command = cmdp[0] };
                case "continuouspantiltmove":
                    return new CmdContinuousPanTiltMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "continuouszoommove":
                    return new CmdContinuousZoomMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "continuousfocusmove":
                    return new CmdContinuousFocusMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "continuousirismove":
                    return new CmdContinuousIrisMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "continuousbrightnessmove":
                    return new CmdContinuousBrightnessMove(cmdp[1], tvparams) { Command = cmdp[0] };
                case "gotoserverpresetname":
                    return new CmdGotoServerPresetName(cmdp[1], tvparams) { Command = cmdp[0] };
                case "gotoserverpresetno":
                    return new CmdGotoServerPrestNo(cmdp[1], tvparams) { Command = cmdp[0] };
                case "gotodevicepreset":
                    return new CmdGotoDevicePreset(cmdp[1], tvparams) { Command = cmdp[0] };
                case "speed":
                    return new CmdSpeed(cmdp[1], tvparams) { Command = cmdp[0] };
                case "ircutfilter":
                    return new CmdIrCutFilter(cmdp[1], tvparams) { Command = cmdp[0] };
                case "backlight":
                    return new CmdBackLight(cmdp[1], tvparams) { Command = cmdp[0] };
            }
            return null;
        }
        public ExecuteCommand OnExecute = null;
        public string Command { get; protected set; }
        public abstract string GetCommandUri(Control container);
        public virtual void CreateUI(Control container)
        {
            ReadParams();
            Button btn = new Button();
            btn.Text = UIResources.BTN_RUN;
            container.Controls.Add(btn);
            btn.Left = container.Width - (btn.Width + 16);
            btn.Top = (container.Height - btn.Height) / 2;
            btn.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            btn.Click += new EventHandler(RunClick);
        }
        protected abstract void ReadParams();
        protected string GetParamValue(string param)
        {
            string[] parts = param.Split('.');
            int ix = 0;
            TreeNodeCollection tnparams = _tvparams.Nodes[0].Nodes;
            while (ix < parts.Length)
            {
                TreeNode node = tnparams[parts[ix]];
                if (node == null)
                {
                    return null;                    
                }
                ParamDefinition pd = node.Tag as ParamDefinition;
                if (pd != null)
                {
                    return pd.Value;
                }
                ix++;
                tnparams = node.Nodes;
            }
            return null;
        }
        private void RunClick(object sender, EventArgs e)
        {
            if (OnExecute != null)
            {
                OnExecute(this);
            }
        }
    }
    public delegate void ExecuteCommand(CmdBase cmd);
}
