﻿using System.Windows.Forms;
using System.Drawing;
using VAPIXProtocol.Properties;

namespace VAPIXProtocol.Commands
{
    public class CmdContinuousPanTiltMove : CmdBase
    {
        private const string cP = "uPan";
        private const string cT = "uTilt";

        public CmdContinuousPanTiltMove(string p, TreeView tvparams)
            : base(p, tvparams)
        {
        }
        public override string GetCommandUri(Control container) 
        {
            int pans = 0;
            int tilts = 0;
            foreach (Control c in container.Controls)
            {
                NumericUpDown ud = c as NumericUpDown;
                if (ud != null)
                {
                    switch (ud.Name)
                    {
                        case cP:
                            pans = (int)ud.Value;
                            break;
                        case cT:
                            tilts = (int)ud.Value;
                            break;
                    }                    
                }
            }
            return "axis-cgi/com/ptz.cgi?" + Command + "=" + pans.ToString() + "," + tilts.ToString();
        }
        public override void CreateUI(Control container)
        {
            base.CreateUI(container);
            Label lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = UIResources.LAB_PANSPEED;
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = 16;
            lbl.Top = (container.Height - lbl.Height) / 2;
            NumericUpDown ud = new NumericUpDown();
            ud.Name = cP;
            ud.Minimum = -100;
            ud.Maximum = 100;
            ud.Value = 0;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (container.Height - ud.Height) / 2;
            lbl = new Label();
            lbl.AutoSize = true;
            lbl.Text = UIResources.LAB_TILTSPEED;
            lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
            container.Controls.Add(lbl);
            lbl.Left = ud.Right + 16;
            lbl.Top = (container.Height - lbl.Height) / 2;
            ud = new NumericUpDown();
            ud.Name = cT;
            ud.Minimum = -100;
            ud.Maximum = 100;
            ud.Value = 0;
            container.Controls.Add(ud);
            ud.Left = lbl.Right + 4;
            ud.Top = (container.Height - ud.Height) / 2;
        }
        protected override void ReadParams()
        {
        }
    }
}
