﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Drawing;
using System.Windows.Forms;
using System.Net;
using VAPIXProtocol.Properties;

namespace VAPIXProtocol
{
    public class ParamDefinition
    {
        private string _value = "";

        public ParamDefinition(XmlElement elm)
        {
            Name = elm.GetAttribute("name");
            NiceName = string.IsNullOrEmpty(elm.GetAttribute("niceName")) ? Name : elm.GetAttribute("niceName");
            _value = elm.GetAttribute("value");
            foreach (XmlElement ch in elm.ChildNodes)
            {
                if (string.Compare(ch.Name, "type", true) == 0)
                {
                    string ro = ch.GetAttribute("readonly");
                    if (!string.IsNullOrEmpty(ro))
                    {
                        ReadOnly = true;
                    }
                    foreach (XmlElement tp in ch.ChildNodes)
                    {
                        DataType = tp.Name;
                        if (string.Compare(tp.Name, "string", true) == 0)
                        {
                            string ml = tp.GetAttribute("maxlen");
                            if (!string.IsNullOrEmpty(ml))
                            {
                                MaxLen = int.Parse(ml);
                            }
                            else
                            {
                                MaxLen = 0;
                            }
                        }
                        else if (string.Compare(tp.Name, "bool", true) == 0)
                        {
                            TrueValue = tp.GetAttribute("true");
                            FalseValue = tp.GetAttribute("false");
                        }
                        else if (string.Compare(tp.Name, "int", true) == 0)
                        {
                            string max = tp.GetAttribute("max");
                            if (!string.IsNullOrEmpty(max))
                            {
                                MaxValue = int.Parse(max);
                            }
                            else
                            {
                                MaxValue = int.MaxValue;
                            }
                            string min = tp.GetAttribute("min");
                            if (!string.IsNullOrEmpty(min))
                            {
                                MinValue = int.Parse(min);
                            }
                            else
                            {
                                MinValue = int.MinValue;
                            }
                        }
                        else if (string.Compare(tp.Name, "enum", true) == 0)
                        {
                            AllowedValues = new List<KeyValuePair<string, string>>();
                            foreach (XmlElement ev in tp.ChildNodes)
                            {
                                if (string.Compare(ev.Name, "entry", true) == 0)
                                {
                                    AllowedValues.Add(new KeyValuePair<string, string>(ev.GetAttribute("value"), string.IsNullOrEmpty(ev.GetAttribute("niceValue")) ? ev.GetAttribute("value") : ev.GetAttribute("niceValue")));
                                }
                            }
                        }
                    }
                }
            }
        }
        public SaveParameter OnSave = null;
        public string Name { get; private set; }
        public string NiceName { get; private set; }
        public string DataType { get; set; }
        public bool ReadOnly { get; set; }
        public string TrueValue { get; set; }
        public string FalseValue { get; set; }
        public List<KeyValuePair<string, string>> AllowedValues { get; set; }
        public int MinValue { get; set; }
        public int MaxValue { get; set; }
        public int MaxLen { get; set; }
        public TreeNode Node { get; set; }
        public string FullPath { get; set; }
        public string Value
        {
            get
            {
                return _value;
            }
            set
            {
                if (!ReadOnly)
                {

                }
            }
        }
        public void CreateEditor(Control container)
        {
            if (ReadOnly)
            {
                Label lbl = new Label();
                lbl.AutoSize = true;
                lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
                lbl.Text = _value;
                container.Controls.Add(lbl);
                lbl.Left = 16;
                lbl.Top = (container.Height - lbl.Height) / 2;
            }
            else
            {
                Button btn = new Button();
                btn.Text = UIResources.BTN_SAVE;
                container.Controls.Add(btn);
                btn.Left = container.Width - (btn.Width + 16);
                btn.Top = (container.Height - btn.Height) / 2;
                btn.Anchor = AnchorStyles.Right | AnchorStyles.Top;

                if (string.Compare(DataType, "string", true) == 0)
                {
                    btn.Click += new EventHandler(SaveTextClick);
                    Label lbl = new Label();
                    lbl.AutoSize = true;
                    lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
                    lbl.Text = UIResources.LAB_VALUE;
                    container.Controls.Add(lbl);
                    lbl.Left = 16;
                    lbl.Top = (container.Height - lbl.Height) / 2;
                    TextBox txb = new TextBox();
                    txb.Text = _value;
                    txb.Name = "txtValue";
                    if (MaxLen > 0)
                    {
                        txb.MaxLength = MaxLen;
                    }
                    container.Controls.Add(txb);
                    txb.Width = (int)((double)container.Width * 0.3);
                    txb.Text = _value;
                    txb.Left = lbl.Right + 4;
                    txb.Top = (container.Height - txb.Height) / 2;
                    txb.Anchor = AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left;
                }
                else if (string.Compare(DataType, "ip", true) == 0)
                {
                    btn.Click += new EventHandler(SaveIPTextClick);
                    Label lbl = new Label();
                    lbl.AutoSize = true;
                    lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
                    lbl.Text = UIResources.LAB_VALUE;
                    container.Controls.Add(lbl);
                    lbl.Left = 16;
                    lbl.Top = (container.Height - lbl.Height) / 2;
                    TextBox txb = new TextBox();
                    txb.Text = _value;
                    txb.Name = "txtValue";
                    container.Controls.Add(txb);
                    txb.Width = (int)((double)container.Width * 0.3);
                    txb.Text = _value;
                    txb.Left = lbl.Right + 4;
                    txb.Top = (container.Height - txb.Height) / 2;
                    txb.Anchor = AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left;
                }
                else if (string.Compare(DataType, "bool", true) == 0)
                {
                    btn.Click += new EventHandler(SaveBoolClick);
                    CheckBox cb = new CheckBox();
                    cb.AutoSize = true;
                    cb.Font = new Font("Arial", 9f, FontStyle.Bold);
                    cb.Text = NiceName;
                    cb.Name = "cbValue";
                    cb.Left = 16;
                    container.Controls.Add(cb);
                    cb.Top = (container.Height - cb.Height) / 2;
                    cb.Checked = _value == TrueValue;
                }
                else if (string.Compare(DataType, "enum", true) == 0)
                {
                    btn.Click += new EventHandler(SaveEnumClick);
                    Label lbl = new Label();
                    lbl.AutoSize = true;
                    lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
                    lbl.Text = UIResources.LAB_VALUE;
                    container.Controls.Add(lbl);
                    lbl.Left = 16;
                    lbl.Top = (container.Height - lbl.Height) / 2;
                    ComboBox cb = new ComboBox();
                    cb.Name = "cbValue";
                    cb.DropDownStyle = ComboBoxStyle.DropDownList;
                    foreach (KeyValuePair<string, string> s in AllowedValues)
                    {
                        cb.Items.Add(s.Value);
                        if (s.Key == _value)
                        {
                            cb.SelectedItem = s.Value;
                        }
                    }
                    container.Controls.Add(cb);
                    cb.Left = lbl.Right + 4;
                    cb.Width = (int)((double)container.Width * 0.3);
                    cb.Anchor = AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Left;
                    cb.Top = (container.Height - cb.Height) / 2;
                }
                else if (string.Compare(DataType, "int", true) == 0)
                {
                    btn.Click += new EventHandler(SaveIntClick);
                    Label lbl = new Label();
                    lbl.AutoSize = true;
                    lbl.Font = new Font("Arial", 9f, FontStyle.Bold);
                    lbl.Text = UIResources.LAB_VALUE;
                    container.Controls.Add(lbl);
                    lbl.Left = 16;
                    lbl.Top = (container.Height - lbl.Height) / 2;
                    NumericUpDown ud = new NumericUpDown();
                    ud.Name = "udValue";
                    ud.Minimum = (decimal)MinValue;
                    ud.Maximum = (decimal)MaxValue;
                    try
                    {
                        ud.Value = Convert.ToDecimal(_value);
                    }
                    catch
                    {
                        ud.Value = ud.Minimum;
                    }
                    container.Controls.Add(ud);
                    ud.Left = lbl.Right + 4;
                    ud.Width = 100;
                    ud.Top = (container.Height - ud.Height) / 2;
                    lbl = new Label();
                    lbl.AutoSize = true;
                    lbl.Text = "(" + MinValue.ToString() + ", " + MaxValue.ToString() + ")";
                    container.Controls.Add(lbl);
                    lbl.Left = ud.Right + 4;
                    lbl.Top = (container.Height - lbl.Height) / 2;
                }
            }
        }
        private void SaveTextClick(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                foreach (Control c in btn.Parent.Controls)
                {
                    TextBox tb = c as TextBox;
                    if ((tb != null) && (tb.Name == "txtValue"))
                    {
                        _value = tb.Text;
                    }
                }
                if (OnSave != null)
                {
                    OnSave(this);
                }
            }
        }
        private void SaveIPTextClick(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                foreach (Control c in btn.Parent.Controls)
                {
                    TextBox tb = c as TextBox;
                    if ((tb != null) && (tb.Name == "txtValue"))
                    {
                        IPAddress ip;
                        if (IPAddress.TryParse(tb.Text, out ip))
                        {
                            _value = tb.Text;
                        }
                    }
                }
                if (OnSave != null)
                {
                    OnSave(this);
                }
            }
        }
        private void SaveBoolClick(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                foreach (Control c in btn.Parent.Controls)
                {
                    CheckBox cb = c as CheckBox;
                    if ((cb != null) && (cb.Name == "cbValue"))
                    {
                        if (cb.Checked)
                        {
                            _value = TrueValue;
                        }
                        else
                        {
                            _value = FalseValue;
                        }
                    }
                }
                if (OnSave != null)
                {
                    OnSave(this);
                }
            }
        }
        private void SaveEnumClick(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                foreach (Control c in btn.Parent.Controls)
                {
                    ComboBox cb = c as ComboBox;
                    if ((cb != null) && (cb.Name == "cbValue"))
                    {
                        foreach (KeyValuePair<string, string> s in AllowedValues)
                        {
                            if (s.Value == cb.SelectedItem.ToString())
                            {
                                _value = s.Key;
                                break;
                            }
                        }
                    }
                }
                if (OnSave != null)
                {
                    OnSave(this);
                }
            }
        }
        private void SaveIntClick(object sender, EventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                foreach (Control c in btn.Parent.Controls)
                {
                    NumericUpDown ud = c as NumericUpDown;
                    if ((ud != null) && (ud.Name == "udValue"))
                    {
                        _value = ud.Value.ToString();
                    }
                }
                if (OnSave != null)
                {
                    OnSave(this);
                }
            }
        }
    }
    public delegate void SaveParameter(ParamDefinition pd);
}
