﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace WatcherCommons.Interfaces
{
    /// <summary>
    /// EventArgs object to pass images to the application
    /// </summary>
    public class FrameEventArgs : EventArgs
    {
        public FrameEventArgs(Bitmap bmp)
        {
            Image = bmp;
        }
        public Bitmap Image { get; private set; }
    }
    public delegate void NewFrameEventHandler(object sender, FrameEventArgs e);
    /// <summary>
    /// Protocol to communicat with the cameras
    /// </summary>
    public interface IWatcherCamera
    {
        /// <summary>
        /// Event fired when a new frame is ready
        /// </summary>
        event NewFrameEventHandler OnNewFrame;
        /// <summary>
        /// Frame size
        /// </summary>
        Size FrameSize { get; }
        /// <summary>
        /// string with the connection settings: url, userName and password
        /// </summary>
        string ConnectionString { get; set; }
        /// <summary>
        /// User name to login
        /// </summary>
        string UserName { get; set; }
        /// <summary>
        /// Password to login
        /// </summary>
        string Password { get; set; }
        /// <summary>
        /// URI of the camera
        /// </summary>
        string Uri { get; set; }
        /// <summary>
        /// Maximum capture rate, in frames per second
        /// </summary>
        int MaxFPS { get; set; }
        /// <summary>
        /// State of the camera, true if capturing, false if stopped
        /// </summary>
        bool Status { get; }
        /// <summary>
        /// Interface to interact with the camera settings system
        /// </summary>
        ICameraSetupManager SetupManager { get; }
        /// <summary>
        /// Implement to perform initialization tasks
        /// </summary>
        void Initialize();
        /// <summary>
        /// Show the camera settings dialog box
        /// </summary>
        /// <param name="parent"></param>
        void ShowCameraConfiguration(Form parent);
        /// <summary>
        /// Start capturing images
        /// </summary>
        void Start();
        /// <summary>
        /// Stop capturing images
        /// </summary>
        void Close();
    }
}
