const Promise = require('promise');
const fs = require('fs');
const path = require('path');
const util = require('util');

function readLinesFromFile (fn) {
  const data = fs.readFileSync(fn, 'utf8');
  return data.trim().split('\n');
}

function readLastLineFromFile (fn) {
  const lines = readLinesFromFile(fn);
  return lines.slice(-1)[0];
}

function readFileAsJson (fn) {
  const data = fs.readFileSync(fn, 'utf8');
  return JSON.parse(data);
}

function buildResolver (resolver, entities) {
  if (typeof resolver === 'string') {
    resolver = eval(resolver);
  } else if (resolver && typeof resolver !== 'function') {
    entities = resolver;
    resolver = undefined;
  }

  if (!resolver) {
    if (entities) {
      resolver = () => entities;
    } else {
      resolver = () => [];
    }
  }

  return resolver;
}

class SmartHomeClient {
  constructor (config) {
    this.loggedIn = false;
    this.shcSerial = config.shcSerial || '123456789';
    this.expires = config.expires || 172800;
    this.locationResolver = buildResolver(config.locationResolver, config.locations);
    this.deviceResolver = buildResolver(config.deviceResolver, config.devices);
    this.capabilityResolver = buildResolver(config.capabilityResolver, config.capabilities);
    this.stateResolver = buildResolver(config.stateResolver, config.states);
  }

  getAllLocations () {
    const value = this.locationResolver();
    return Promise.resolve(value);
  }

  getAllDevices () {
    const value = this.deviceResolver();
    return Promise.resolve(value);
  }

  getAllCapabilities () {
    const value = this.capabilityResolver();
    return Promise.resolve(value);
  }

  getAllCapabilityStates () {
    const value = this.stateResolver();
    return Promise.resolve(value);
  }

  authorizeWithRefresh (refreshToken) {
    this.loggedIn = true;
    return Promise.resolve({
      refresh: refreshToken,
      expires: this.expires,
      shc: { serialnumber: this.shcSerial }
    });
  }

  authorize (code) {
    return authorizeWithRefresh(code);
  }
}

module.exports = SmartHomeClient;