const Promise = require('promise');

Array.prototype.first = function () {
  return this[0];
};

function getEntityName (entity) {
  return entity.Config.filter(function (config) {
    return config.name === 'Name';
  }).map(function (config) {
    return config.value;
  }).first();
}

function transformDevice (device) {
  return {
    id: device.id,
    manufacturer: device.manufacturer,
    name: getEntityName(device),
    description: device.desc,
    serialnumber: device.serialnumber,
    type: device.type
  };
}

const SmartHome = function (config) {
  const self = this;
  const Client = require('./shcs/' + config.type + '.js');
  const client = new Client(config);

  function finish (login) {
    return login.then(function (data) {
      return Promise.all([
        client.getAllLocations().then(function (locations) {
          self.locations = locations;
        }),
        client.getAllDevices().then(function (devices) {
          self.devices = devices;
        }),
        client.getAllCapabilities().then(function (capabilities) {
          self.capabilities = capabilities;
        })
      ]).then(function () {
        return data; 
      });
    });
  }

  self.refresh = function (code) {      
    return finish(client.authorizeWithRefresh(code));
  };
  self.login = function (code) {      
    return finish(client.authorize(code));
  };
  self.getAllStates = function (name) {
    return client.getAllCapabilityStates().then(function (capabilities) {
      return capabilities.map(function (capability) {
        const device = self.getDevice(capability.Id);
        const location = self.getLocations(device).first();
        const states = capability.State.filter(function (state) {
          return state.name === name;
        });

        return {
          id: capability.Id,
          location: location,
          device: transformDevice(device),
          state: states.first()
        };
      }).filter(function (capability) {
        return capability.state !== undefined;
      });
    });
  };
  self.getDevice = function (capabilityId) {
    const id = '/capability/' + capabilityId;

    return self.devices.filter(function (device) {
      return device.Capabilities.filter(function (capability) {
        return capability.value === id;
      }).length > 0;
    }).first();
  };
  self.getLocations = function (device) {
    if (device && device.Location) {
      return device.Location.map(function (location) {
        const id = location.value.substr('/location/'.length);

        return self.locations.filter(function (location) {
          return location.id === id;
        }).map(getEntityName).first();
      });
    }

    return [];
  };
};

module.exports = SmartHome;